/*
 *  This file is part of Dascant.
 *
 *  Dascant is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dascant is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dascant; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 *  USA
 */

/**
 * The cucumber class.  This is a closed quadric surface symmetric along
 * four orthogonal axes.  It is the tetraspace analog of the ellipsoid in
 * 3-space.
 */
#include <cucumber.h>

/**
 * Constructor allowing specification of the semiaxes as a vector.
 */
cucumber::
cucumber( const vector4 *s )
	: quadric()
{
	initialize(s);
}

/**
 * Constructor for a glome of radius r.
 */
cucumber::
cucumber( double r )
{
	vector4 v(r, r, r, r);
	initialize(&v);
}

/**
 * Constructor allowing specification of the semiaxes as four doubles.
 */
cucumber::
cucumber( double x, double y, double z, double w )
{
	vector4 v(x, y, z, w);
	initialize(&v);
}

/**
 * Constructor for the unit sphere.
 */
cucumber::
cucumber(void)
	: quadric()
{
	vector4 v(1, 1, 1, 1);
	initialize(&v);
}

/**
 * Create an cucumber with the given dimensions.  It is centered on the
 * origin, and its axes are the coordinate axes.
 */
void cucumber::
initialize( const vector4 *s )
{
	int i;

	for (i = 0; i < 15; ++i)
		coeff[term[i][0]][term[i][1]][term[i][2]][term[i][3]] = 0;

	size = *s;

	coeff[0][0][0][0] = -1;
	coeff[2][0][0][0] = 1/(s->elem[0]*s->elem[0]);
	coeff[0][2][0][0] = 1/(s->elem[1]*s->elem[1]);
	coeff[0][0][2][0] = 1/(s->elem[2]*s->elem[2]);
	coeff[0][0][0][2] = 1/(s->elem[3]*s->elem[3]);
}

/** 
 * Reinitialize the cucumber using the new dimensions.
 */
cucumber &cucumber::operator
()( const vector4 *s )
{
	initialize(s);
	return *this;
}
