/*
 *  This file is part of Dascant.
 *
 *  Dascant is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dascant is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dascant; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 *  USA
 */

/**
 * The entity.  A mostly abstract class representing, essentially, stuff
 * that rays can collide with.
 */
#include <entity.h>

/**
 * Default constructor.  Sets the default color, but naught else.
 */
entity::
entity()
{
	red = DEFAULT_RED;
	green = DEFAULT_GREEN;
	blue = DEFAULT_BLUE;
}

int entity::
detectCollisions( const ray *R, collision *C ) const
{
	return 0;
}

void entity::
translate( const vector4 &c )
{
}

void entity::
rotate( axis u, axis v, double t )
{
}

/**
 * Return the number of atomic entities in this entity.
 */
int entity::
size( void ) const
{
	return 1;
}

/**
 * determine whether the given point is inside this entity.
 */
bool entity::
inside( const vector4 &v ) const
{
	return false;
}

char *entity::
toString( char *s ) const
{
	static char buf[6];
	if (!s) s = buf;
	strcpy(s, "thing");
	return s;
}

void entity::
setColor( unsigned char r, unsigned char g, unsigned char b )
{
	red = r;
	green = g;
	blue = b;
}

void entity::
setColor( unsigned int c )
{
	blue = c & 0xff;
	c >>= 8;

	green = c & 0xff;
	c >>= 8;

	red = c & 0xff;
	c >>= 8;
}
