/*
 *  This file is part of Dascant.
 *
 *  Dascant is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dascant is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dascant; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 *  USA
 */

#ifndef _DASCANT_CAMERA_H
#define _DASCANT_CAMERA_H

/**
 * The camera class.  A camera keeps track of its own rays, for the purpose
 * of raytracing.  Repositioning the camera causes new rays to be generated
 * (but not traced).
 */

#include <vector4.h>
#include <entity.h>
#include <ray.h>
#include <rotation4.h>
#include <frame.h>

class camera {
private:
	vector4 nextPosition;
	bool updatePosition;
	rotation4 nextOrientation;
	bool updateOrientation;
	void initialize(int w, int h, double d);
public:
	vector4 position;
	rotation4 orientation;
	vector4 pointing;
	frame *view;
	ray **rays;
	unsigned char *pixel;
	int width, height;
#if 0
	Transform4 orientation;
#endif
	double depth;
	camera(void);
	camera(int w, int h, double d);
	~camera();
	camera *copy(void);
	void setPosition(double x, double y, double z, double w);
	void setPosition(const vector4 *V, const rotation4 *R);
	void enqueuePosition(const vector4 *V);
	void enqueueOrientation(const rotation4 *R);
	void compute_rays(void);
	void snapshot(const entity *e);
};

#endif
