/*
 *  This file is part of Dascant.
 *
 *  Dascant is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dascant is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dascant; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 *  USA
 */

#ifndef _DASCANT_COLLISION_H
#define _DASCANT_COLLISION_H

#include <entity.h>
#include <vector4.h>
#include <quadric.h>

class collision {
private:
	void initialize(const entity *e);
	bool operator <(collision &);

public:
	unsigned char red, green, blue;
	vector4 location;
	vector4 incoming;
	vector4 outgoing;
	vector4 normal;
	double index;
	bool inside;
	const entity *object;
	const quadric *surface;
	double rayPosition;
	int increment; /* 1 for entry, -1 for exit, 0 for osculation */

	collision(void);
	collision(const entity *e);
	collision(const ray *R, double t, const entity *e);
	char *toString(char *s=0x0);
	collision &operator()(const ray *, double, const entity *);
	void computeNormal(void);
};

int collisionCmp   (const collision *, const collision *);
int collisionPtrCmp(const collision **, const collision **);

#endif
