/*
 *  This file is part of Dascant.
 *
 *  Dascant is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dascant is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dascant; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 *  USA
 */

#ifndef _DASCANT_FRAME_H
#define _DASCANT_FRAME_H

#include <vector4.h>
#include <config.h>

class frame {
private:
	/* image of a local frame of reference under the given rotation through
	 * ROTATE_STEPSIZE.  Note that these vectors are still in the local
	 * reference frame.
	 * Storage: image[u][v][0][n] is the image of the nth axis under a
	 * rotation around the uv-plane.  image[u][v][1][n] is the same, rotated
	 * around the -uv-plane.  */
	static vector4 image[4][4][2][4];
	static bool ran_setup;
public:
	vector4 origin;
	vector4 local[4];

	frame(void);

	void move(const vector4 &);
	void rotate(int U, int V, int s=0);

	vector4 toAbsolute(const vector4 &);
	vector4 toAbsoluteNoTranslation(const vector4 &);
	vector4 fromAbsolute(const vector4 &);
};

#endif
