/*
 *  This file is part of Dascant.
 *
 *  Dascant is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dascant is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dascant; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 *  USA
 */

#ifndef _DASCANT_QUADRIC_H
#define _DASCANT_QUADRIC_H

/**
 * The Quadric class.  Objects of this type represent four-dimenional
 * quadric surfaces.
 */

#include "ray.h"
#include "entity.h"

class Term;
class collision;

class quadric : public entity {
private:
	/**
	 * Surface orientation.  The form of the quadric solid is nuP;(x,y,z,w)
	 * &lt;= 0.  If this value is 1, then, the surface will be oriented in
	 * the normal way.  Otherwise, the solid described will be the
	 * complement of the standard solid you'd expect (such as an cucumberal
	 * hole instead of an cucumber).
	 */
	int normalOrientation;
public:
	static const int term[][4];
	static const int degree[];
	double coeff[3][3][3][3];

	quadric(void);
	double getCoefficient(Term trm);
	void setCoefficient(double c, Term trm);
	void add(Term T);
	void add(quadric &Q);
	int toString(char *);
	void invert(void);

	inline double (*operator[](int n))[3][3] { return coeff[n]; }

	virtual int detectCollisions(const ray *R, collision *) const;
	virtual bool inside(const vector4 &) const;
	virtual char *toString(char *s=0x0) const;
	virtual void translate(const vector4 &);
	virtual void rotate(axis, axis, double);

	void setComplement(bool=true);
};

#endif
