/*
 *  This file is part of Dascant.
 *
 *  Dascant is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dascant is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dascant; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 *  USA
 */

#ifndef _DASCANT_VECTOR4_H
#define _DASCANT_VECTOR4_H
/**
 * The vector4 class.  This class describes a four-dimensional vector in
 * homogeneous coordinates (though the user can ignore this feature as every
 * vector defaults to being a point).
 */


class vector4 {
public:
	double elem[4];
	vector4(void);
	vector4(const vector4 &V);
	vector4(double X, double Y, double Z);
	vector4(double X, double Y, double Z, double W);
	vector4(double a[], int=4);
	void toArray(double []);
	vector4 *copy(void);
	char *toString(char *s=0x0);
	int fromString(const char *);
	
	double norm(void);
	void normalize(void);
	double dot(const vector4 &V) const;
	vector4 &cross(const vector4 &, const vector4 &);
	double det33(const vector4 &U, const vector4 &V, int exclude);

	vector4 operator *(double);
	vector4 operator =(const vector4 &);
	vector4 operator ()(double, double, double, double);
	vector4 operator ()(double[]);
	vector4 operator +(vector4 &);
	vector4 operator +=(vector4 &);
	inline double &operator[](int n) { return elem[n]; }
};

#endif
