/*
 *  This file is part of Dascant.
 *
 *  Dascant is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dascant is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dascant; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 *  USA
 */

#include <dascant.h>
#include <gui.h>
#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>
#include <math.h>
#include <gtk/gtk.h>

int siz = SIZE;
entity *scene;
bool anim = false;

void
dascant_run( camera *C )
{
	struct timeval then, now;
	double diff;
	char fps_text[32];
	int tframes=0;

	gettimeofday(&then, NULL);
	while(anim) {
#ifndef MUTLTITHREADED
		while(gtk_events_pending())
			gtk_main_iteration();
#endif

		C->snapshot(scene);

		gettimeofday(&now, NULL);
		++tframes;
		diff = now.tv_sec - then.tv_sec;
		diff += (now.tv_usec - then.tv_usec) * 1e-6;
		if (diff > 1) {
			then = now;
			sprintf(fps_text, "%5.1f fps", tframes/diff);
			printf("%s\n\033[1A", fps_text);
			tframes = 0;
		}

		gui_update();
	}
}

void
world_init( camera **cam )
{
	static cucumber G;
	static tesseract T;
	static quadric S1, S2;
	static glue S(2), World(3);
	static tesseract D;
	vector4 v;

	/* make a camera */
	*cam = new camera(siz, siz, 200.0);
	(*cam)->view = new frame();
	(*cam)->view->origin(0, 0, 10, 0);
	(*cam)->compute_rays();

	/* make a red glome at (-3, 0, 0, 0) */
	G.setColor(0xff, 0x00, 0x00);
	G.translate(v(-3, 0, 0, 0));

	/* make a green tesseract */
	T.setColor(0x00, 0xff, 0x00);

	/* make a blue sphone at (+3, 0, 0, 0) */
	S1[2][0][0][0] = 2;
	S1[0][2][0][0] = -1;
	S1[0][0][2][0] = 1;
	S1[0][0][0][2] = 1;
	S1.translate(v(3, +1, 0, 0));
	S.add(&S1);
	S2[0][2][0][0] = 1;
	S2[0][0][0][0] = -1;
	S.add(&S2);
	S.combine_mode = glue::INTERSECTION;

	/* glue the glome, tesseract, and sphone together */
	World.add(&G);
	World.add(&T);
	World.add(&S);

	/* make a die */
	static glue Die(5);
	static tesseract D1;
	static cucumber D2(0.25), D3(0.25), D4(0.25);
	static cucumber D5(10);
	D1.setColor(0xA0, 0xA0, 0xA0);
	D2.setColor(0xFF, 0x00, 0x00);
	D3.setColor(0x00, 0xFF, 0x00);
	D4.setColor(0x00, 0x00, 0xFF);
	D2.translate(v(-1, 0, 0, 0));
	D3.translate(v(0, 1, 0, 0));
	D4.translate(v(0, 0, 1, 0));
	D5.setColor(0x37d3b6);
	D5.setComplement();
	Die.add(&D1);
	Die.add(&D2);
	Die.add(&D3);
	Die.add(&D4);
	//Die.add(&D5);

	/* decide what's in the scene */
	scene = &World;
}

int
main( int argc, char *argv[] )
{
	char c;
	camera *cam;

#ifdef MULTITHREADED
	g_thread_init(NULL);
	gdk_threads_init();
#endif
	gtk_init(&argc, &argv);

	while ((c = getopt(argc, argv, "ahs:")) != EOF)
		switch(c) {
		case 'h': case '?':
			fprintf(stderr, "usage: %s [-s size] [filename]\n", *argv);
			return 1;
		case 's':
			siz = strtol(optarg, NULL, 0);
			break;
		case 'a':
			anim = true;
			break;
		}

	if (!rcfile::load()) return 0;

	world_init(&cam);
	gui_init(cam);

#ifndef MULTITHREADED
	if (anim)
		dascant_run(cam);
	else {
		cam->snapshot(scene);
		gtk_main();
	}
#else
	GError *dascant_err, *gui_err;
	GThread *dascant_tid, *gui_tid;
	dascant_tid = g_thread_create((GThreadFunc)dascant_run, cam,
		1, &dascant_err);
	gui_tid  = g_thread_create((GThreadFunc)gtk_main, NULL,
		1, &gui_err);
	g_thread_join(gui_tid);
#endif

	return 0;
}
