/*
 *  This file is part of Dascant.
 *
 *  Dascant is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dascant is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dascant; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 *  USA
 */

/**
 * Rcfile class.  Represents data found in a $HOME/.dascantrc.
 */
#include <rcfile.h>
#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <config.h>
#include <math.h>

bool rcfile::
load( void )
{
	FILE *fp;
	char buf[256], filename[FILENAME_MAX], *cp;
	int line=0;

	sprintf(filename, "%s/.dascantrc", getenv("HOME"));
	fp = fopen(filename, "r");
	if (!fp) {
		if (errno == ENOENT) return 1;
		perror(filename);
		return 0;
	}

	while(fgets(buf, sizeof(buf), fp)) {
		++line;
		cp = buf;
		while(*cp != '\n' && *cp != '\r' && isspace(*cp)) ++cp;
		if (*cp == '\r' || *cp == '\n' || *cp == '#') continue;

		if (!strncmp(cp, "move", 4)) {
			cp += 4;
			move_stepsize = strtod(cp, NULL);
			if (move_stepsize == 0.0) {
				fprintf(stderr, "%s:%d: Need a real number.\n%s",
					filename, line, buf);
				for (int i = 0; i < cp-buf; ++i)
					fputc(' ', stderr);
				fprintf(stderr, "^\n");
			}
			continue;
		}

		if (!strncmp(cp, "rotate", 6)) {
			cp += 6;
			rotate_stepsize = M_PI*strtod(cp, NULL)/180.0;
			if (rotate_stepsize == 0.0) {
				fprintf(stderr, "%s:%d: Need a real number.\n%s",
					filename, line, buf);
				for (int i = 0; i < cp-buf; ++i)
					fputc(' ', stderr);
				fprintf(stderr, "^\n");
			}
			continue;
		}

		if (!strncmp(cp, "light", 5)) {
			cp += 5;
			if (lightSource.fromString(cp) != 4)
				fprintf(stderr, "%s:%d: Need a 4-vector.\n%s",
					filename, line, buf);
			continue;
		}
	}

	fclose(fp);

	return true;
}

bool rcfile::
save( void )
{
	FILE *fp;
	char filename[FILENAME_MAX];

	sprintf(filename, "%s/.dascantrc", getenv("HOME"));
	fp = fopen(filename, "w");
	if (!fp) {
		perror(filename);
		return false;
	}

	fprintf(fp, "# This file created automatically by Dascant.\n\n");
	fprintf(fp, "move %.4f\nrotate %.4f\nlight %s\n",
		move_stepsize,
		180.0*rotate_stepsize/M_PI,
		lightSource.toString());

	fclose(fp);

	return true;
}

double rcfile::move_stepsize = MOVE_STEPSIZE;
double rcfile::rotate_stepsize = ROTATE_STEPSIZE;
vector4 rcfile::lightSource;
