import java.text.NumberFormat;

public class Collision implements Comparable<Collision> {
	public Vector4D location;
	public Vector4D incoming;
	public Vector4D outgoing;
	public Vector4D normal;
	public double index;
	public boolean inside;
	public Entity object;
	public double rayPosition;
	public boolean entry;

	private void initialize( Entity e )
	{
		location = null;
		incoming = null;
		outgoing = null;
		inside = false;
		object = e;
		rayPosition = 0;
		entry = true;
	}

	public Collision( Entity e ) {
		initialize(e);
	}

	/**
	 * Class constructor with ray.
	 * @param R the ray causing the collision.
	 * @param t the distance along the ray that the collision occured.
	 * @param e the object the ray struck.
	 */
	public Collision( Ray R, double t, Entity e )
	{
		initialize(e);
		location = R.evalAt(t);
		rayPosition = t;
	}

	public String toString()
	{
		String s = "[ " + location;
		NumberFormat fmt = NumberFormat.getInstance();
		fmt.setMaximumFractionDigits(4);
		if (rayPosition != 0)
			s += ", t = " + fmt.format(rayPosition);
		return s + (entry ? "*" : "") + " ]";
	}

	public int compareTo( Collision other )
	{
		if (rayPosition > other.rayPosition)
			return 1;
		else if (other.rayPosition > rayPosition)
			return -1;
		else
			return 0;
	}
};
