/**
 * The DSML_Exception class.  Should be used in place of Exception for
 * errors in compiling a DSML file.  Before creating any instances, the
 * static method <tt>registerLexer</tt> should be called to indicate where
 * we should be getting line numbers from.
 */
public class DSML_Exception extends Exception {
	/**
	 * The DSML_Lexer to get line numbers from.
	 */
	private static DSML_Lexer lexer;

	/**
	 * Register a DSML_Lexer to get line numbers from.
	 * @param L the DSML_Lexer to use.
	 */
	public static void registerLexer( DSML_Lexer L )
	{
		lexer = L;
	}

	/**
	 * Class constructor for a miscellaneous error.
	 * @param s An error message.
	 */
	public DSML_Exception( String s )
	{
		super("On line " + lexer.lineno() + ": " + s);
	}

	/**
	 * Class constructor for an "expected this, got that" error.
	 * @param expected what we expected.
	 * @param got what we got.
	 */
	public DSML_Exception( String expected, DSML_Token got )
	{
		super("On line " + lexer.lineno() + ": expected " + expected
			+ ", but got " + got);
	}
};
