/**
 * The Ray class.  For raytracing.
 */

import java.util.HashMap;

public class Ray extends Vector4D {
	//private HashMap<Term, double[]> ttab;
	public double ttab[][];
	private Vector4D R0;

	/**
	 * Class constructor.  Generate a ray from the origin through the point
	 * (<tt>x</tt>,<tt>y</tt>) through the (negative) image plane.
	 * @param C the Camera receiving this ray.
	 * @param X x-coordinate of point on image plane.
	 * @param Y y-coordinate of point on image plane.
	 */
	public Ray( Camera C, double X, double Y )
	{
		super(X, Y, -C.depth);

		normalize();
/*
		Vector4D RR = C.orientation.applyTo(this);
		x = RR.x;
		y = RR.y;
		z = RR.z;
		w = RR.w;
 */

		R0 = C.position;

		double a, b, c;
		double  t[] = new double[2], // coeffs of first parametric term
		       tt[] = new double[2]; // coeffs of second parametric term
		int i, m;
		boolean gott;

		/* build the array of coefficients derived from the parametrized
		 * form of this ray. */
		//ttab = new HashMap(21);
		ttab = new double[15][3];
		for (i = 0; i < Quadric.term.length; ++i) {
			gott = false;

			// figure out what linear terms (in t) will be
			// multiplied together.
			for (m = 0; m < 4; ++m) {
				switch(Quadric.term[i][m]) {
				case 2:
					 t[1] = this.elem[m];
					 t[0] = R0.elem[m];
					tt[1] = this.elem[m];
					tt[0] = R0.elem[m];
					m = 4; // we can break out of the loop now
					break;
				case 1:
					if (gott) {
						tt[1] = this.elem[m];
						tt[0] = R0.elem[m];
						break;
					}
					else {
						t[1] = this.elem[m];
						t[0] = R0.elem[m];
						gott = true;
					}
					break;
				}
			}

			// actually multiply them together
			switch(Quadric.degree[i]) {
			case 0:
				a = 0;
				b = 0;
				c = 1;
				break;
			case 1:
				a = 0;
				b = t[1];
				c = t[0];
				break;
			case 2:
				a = t[1]*tt[1];
				b = t[0]*tt[1] + t[1]*tt[0];
				c = t[0]*tt[0];
				break;
			default:
				/* FIXME: i probably shouldn't just ignore this
				 * possibility */
				return;
			}

			// store the results in the table
			ttab[i][0] = c;
			ttab[i][1] = b;
			ttab[i][2] = a;
		} /* for(...) */
	}

	public Vector4D evalAt( double t )
	{
		return new Vector4D(elem[0]*t + R0.elem[0],
		                    elem[1]*t + R0.elem[1],
		                    elem[2]*t + R0.elem[2],
		                    elem[3]*t + R0.elem[3]);
	}

	public String toString()
	{
		return "[ " + super.toString() + " t + " + R0 + " ]";
	}

	private Ray() {} 

	public Ray copy( Camera C, int X, int Y )
	{
		Ray R = new Ray();

		R.elem[0] = X;
		R.elem[1] = Y;
		R.elem[2] = -C.depth;
		R.elem[3] = 0;

		R.normalize();
		R.R0 = C.position;

		R.ttab = new double[15][3];
		for (int i = 0; i < 15; ++i)
			for (int j = 0; j < 3; ++j)
				R.ttab[i][j] = ttab[i][j];

		return R;
	}
};
