/**
 * The Term class; for convenience in creating quadrics and adding
 * terms.
 */
public class Term {
	public int powX, powY, powZ, powW;
	public double coeff;

	public Term()
	{
		powX = powY = powZ = powW = 0;
		coeff = 1.0;
	}

	public Term( int X, int Y, int Z, int W )
	{
		powX = X;
		powY = Y;
		powZ = Z;
		powW = W;
		coeff = 1.0;
	}

	public int degree()
	{
		return powX + powY + powZ + powW;
	}

	public String toString()
	{
		String cs, xs, ys, zs, ws;

		if (Math.abs(coeff) < 1e-16)
			return "0";

		cs = "" + coeff;

		switch(powX) {
		default: xs = ""; break;
		case 1: xs = "x"; break;
		case 2: xs = "x^2"; break;
		}

		switch(powY) {
		default: ys = ""; break;
		case 1: ys = "y"; break;
		case 2: ys = "y^2"; break;
		}

		switch(powZ) {
		default: zs = ""; break;
		case 1: zs = "z"; break;
		case 2: zs = "z^2"; break;
		}

		switch(powW) {
		default: ws = ""; break;
		case 1: ws = "w"; break;
		case 2: ws = "w^2"; break;
		}

		return cs + xs + ys + zs + ws;
	}

	int getPow( int i )
	{
		switch(i) {
		case 0: return powX;
		case 1: return powY;
		case 2: return powZ;
		case 3: return powW;
		}

		return -1;
	}
};
