/**
 * A class to test just about everything.  Doesn't test parsing DSML files
 * at the moment.
 */
import java.awt.*;
import java.awt.event.*;
import java.awt.image.MemoryImageSource;

public class test_everything {
	/**
	 * The camera to trace rays from.
	 */
	private Camera cam;
	private Vector4D nextPosition = null;
	//private int mouseStartX, mouseStartY;
	//private boolean started;

	private int screenWidth, screenHeight;

	/* the offscreen drawable */
	private Graphics gfx;
	private Image offscreen;
	private boolean animate;
	private MemoryImageSource source;
	private int siz;

	private double sigmoid( double x )
	{
		return 1.0 / (1 + Math.exp(-x));
	}

	/**
	 * Class constructor.  The constructor also runs the test.  The test can
	 * be either on an ellipse or a tesseract; the latter can be specified
	 * by the command-line argument <tt>-t</tt>.  A MouseListener is
	 * created to handle mouse clicks.
	 * @param s the length of the side of the square region of the screen
	 * to draw.
	 */
	public test_everything( int s, Entity ent, boolean anim )
	{
		siz = s;

		/* create the scene */
		Vector4D light = new Vector4D(-1, 1, 0.5, 1);

		cam = new Camera(s, s, 200.0);
		cam.setPosition(0, 0, 10, 0);

		/* initialize the graphical environment */
		GraphicsEnvironment ge =
			GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice gd = ge.getDefaultScreenDevice();
		Frame frame = new Frame(gd.getDefaultConfiguration());
		Window win = new Window(frame);

		win.addKeyListener(new KeyAdapter() {
			/**
	 		* keyboard handler.
	 		*  FIXME: unused... not receiving keyboard events
	 		*/
			public void keyTyped( KeyEvent ev )
			{
				if (ev.getKeyCode() == KeyEvent.VK_ESCAPE)
					System.exit(0);
		
				switch(ev.getKeyChar()) {
				case 'w': cam.position.elem[2] -= 1; break;
				case 's': cam.position.elem[2] += 1; break;
				case 'a': cam.position.elem[0] -= 1; break;
				case 'f': cam.position.elem[0] += 1; break;
				case 'e': cam.position.elem[1] += 1; break;
				case 'd': cam.position.elem[1] -= 1; break;
				}

				cam.compute_rays();
			}
		});

		win.addMouseListener(new MouseAdapter() {
			/**
	 		 * mouse click handler.  Clicking with the left mouse button
	 		 * sets the camera's <i>x</i> and <i>y</i> coordinates.
	 		 * Clicking with the right mouse buttons sets the camera's
	 		 * <i>w</i> and </i>z</i> coordinates.
			*/
			public void mouseClicked( MouseEvent ev )
			{
				switch (ev.getButton()) {
				case MouseEvent.BUTTON1:
					nextPosition = new Vector4D(cam.position);
					nextPosition.elem[0] =  0.05
						* (+ev.getX() - screenWidth /2);
					nextPosition.elem[1] =  0.05
						* (-ev.getY() + screenHeight/2);
					break;
				case MouseEvent.BUTTON3:
					nextPosition = new Vector4D(cam.position);
					nextPosition.elem[3] =  0.05
						* (+ev.getX() - screenWidth /2);
					nextPosition.elem[2] =  0.05
						* (+ev.getY() - screenHeight/2);
					break;
				}
			}
		});

		try {
			/* go to full screen mode */
			gd.setFullScreenWindow(win);
			win.requestFocus();

			/* get screen dimensions */
			screenWidth = win.getWidth();
			screenHeight = win.getHeight();
			int size = screenWidth * screenHeight;

			/* initialize the offscreen drawable a.k.a. backbuffer */
			int pixels[] = new int[size];
			source =
				new MemoryImageSource(screenWidth, screenHeight,
					pixels, 0, screenWidth);
			source.setAnimated(true);
			source.setFullBufferUpdates(true);
			offscreen = Toolkit.getDefaultToolkit().createImage(source);
			animate = anim;

			/* blank the screen */
			gfx = win.getGraphics();
			gfx.setColor(Color.black);
			gfx.fillRect(0, 0, screenWidth, screenHeight);

			/* draw the help message */
			gfx.setColor(Color.red);
			gfx.drawString("Click button 1 to move camera to (x,y)",
				20, screenHeight/2 + 20);
			gfx.drawString("Click button 3 to move camera to (w,z)",
				20, screenHeight/2 + 40);
			gfx.drawString("Command-line arguments:   -t  -s size",
				20, screenHeight/2 + 60);
			gfx.drawString("Light source is at " + light,
				20, screenHeight/2 + 80);

			/* draw axes */
			gfx.drawLine(screenWidth/2, 0, screenWidth/2, screenHeight);
			gfx.drawLine(0, screenHeight/2, screenWidth, screenHeight/2);
			gfx.drawString("x / w", screenWidth/2+siz+20, screenHeight/2+20);
			gfx.drawString("y",     screenWidth/2+20, screenHeight/2-siz-20);
			gfx.drawString("z",     screenWidth/2+20, screenHeight/2+siz+20);

			// draw box
			gfx.setColor(Color.red);
			gfx.drawRect(
				(screenWidth-siz)/2-1, (screenHeight-siz)/2-1, siz+1, siz+1);
			gfx.setColor(Color.black);
			gfx.fillRect(
				(screenWidth-siz)/2, (screenHeight-siz)/2,
				siz,                 siz);

			// for keeping track of fps
			int frames;
			long lastTime = System.currentTimeMillis();

			// set a clipping region
			Rectangle R;
			R = new Rectangle((screenWidth-siz)/2, (screenHeight-siz)/2,
				siz, siz);
			R.add(new Rectangle(0, 0, 100, 40));
			gfx.setClip(R);

			//int total_frames=0;
			//System.out.println("Will bail out after 100 frames.");

			// want vector to light to be normalized
			light.normalize();

			frames = 0;
			while(true) {
				int x, y, X, Y, k;

				frames++;

				// update the camera position (and recalculate rays), if
				// necessary.
				if (nextPosition != null) {
					cam.setPosition(nextPosition);
					nextPosition = null;
				}

				for (x = 0; x < siz; ++x)
					for (y = 0; y < siz; ++y) {
						X = (screenWidth - siz)/2  + x;
						Y = (screenHeight + siz)/2 - y;

						// detect any colisions along the ray through this
						// point on the image plane.
						Collision C[] = ent.detectCollisions(cam.ray[y][x]);
						if (C == null) { // no collisions?
							pixels[Y*screenWidth+X] = 0xff000000;
							continue;
						} else if (C[0].rayPosition > 0)
							k = 0;       // first colision in front?
						else if (C.length > 0 && C[1].rayPosition > 0)
							k = 1;       // second collision in front?
						else {           // all collisions behind?
							pixels[Y*screenWidth+X] = 0xff000000;
							continue;
						}

						// figure out the color accd. to surface normal
						double cosine = C[k].normal.dot(light);
						if (cosine <= 0) continue;
						int b = (int)(255 * cosine);
						int color = 0xff000000 | b;
						pixels[Y*screenWidth+X] = color;
					}

				// draw scene
				source.newPixels((screenWidth-siz)/2,
				                 (screenHeight-siz)/2,
				                 siz, siz);
				gfx.drawImage(offscreen, 0, 0, null);

				// draw fps
				long t = System.currentTimeMillis();
				if (t - lastTime >= 1000) {
					gfx.setColor(Color.black);
					gfx.fillRect(0, 0, 500, 60);
					gfx.setColor(Color.red);
					gfx.drawString(Vector4D.fmt.format(
						1000.0*frames/(t-lastTime)) + " fps", 20, 20);
					gfx.drawString("Camera is at " + cam.getPosition(),
						20, 40);
					frames = 0;
					lastTime = t;
				}

				// bail out after 100 frames
				//if (++total_frames == 100) System.exit(0);
			}
		} catch(Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
	}

	public static void main( String argv[] )
	{
		int siz=128, i;
		Entity ent = null;
		boolean animate = true;

		for (i = 0; i < argv.length; ++i)
			if (argv[i].compareTo("-a") == 0)
				animate = false;
			else if (argv[i].compareTo("-s") == 0)
				siz = Integer.valueOf(argv[++i]).intValue();
			else if (argv[i].compareTo("-p") == 0)
				ent = new Parabolonk();
			else if (argv[i].compareTo("-e") == 0)
				ent = new Ellipsonk();
			else if (argv[i].compareTo("-t") == 0)
				ent = new Tesseract();
			else if (argv[i].compareTo("-q") == 0) {
				Quadric Q = new Quadric();
				Q.coeff[1][0][0][0] = 1;
				Q.coeff[0][0][1][0] = -1;
				ent = Q;
			}

		if (ent == null)
			ent = new Tesseract();

		test_everything f = new test_everything(siz, ent, animate);
	}
};
